#include "nmea0183.h"
#pragma hdrstop

/*
** Author: Samuel R. Blackburn
** Internet: sam_blackburn@pobox.com
**
** You can use it any way you like as long as you don't try to sell it.
**
** Copyright, 1996, Samuel R. Blackburn
**
** $Workfile: zwp.cpp $
** $Revision: 4 $
** $Modtime: 10/10/98 2:38p $
*/

/*
** This Sentence Not Recommended For New Designs
** ZTG is recommended.
*/

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

ZWP::ZWP()
{
   Mnemonic = "ZWP";
   Empty();
}

ZWP::~ZWP()
{
   Mnemonic.Empty();
   Empty();
}

void ZWP::Empty( void )
{
   UTCTimeString.Empty();
   ArrivalTimeString.Empty();
   To.Empty();
}

BOOL ZWP::Parse( const SENTENCE& sentence )
{
   /*
   ** ZWP - Arrival time at waypoint
   **
   **        1         2         3    4
   **        |         |         |    |  
   ** $--ZWP,hhmmss.ss,hhmmss.ss,c--c*hh<CR><LF>
   **
   ** 1) Universal Time Coordinated (UTC)
   ** 2) Arrival Time
   ** 3) Waypoint ID (To)
   ** 4) Checksum
   */

   /*
   ** First we check the checksum...
   */

   if ( sentence.IsChecksumBad( 4 ) == True )
   {
      SetErrorMessage( "Invalid Checksum" );
      return( FALSE );
   }

   CTime temp_time = CTime::GetCurrentTime();

   int year  = temp_time.GetYear();
   int month = temp_time.GetMonth();
   int day   = temp_time.GetDay();

   UTCTimeString = sentence.Field( 1 );

   char temp_number[ 3 ];

   temp_number[ 2 ] = 0x00;

   temp_number[ 0 ] = UTCTimeString[ 0 ];
   temp_number[ 1 ] = UTCTimeString[ 1 ];

   int hours = ::atoi( temp_number );

   temp_number[ 0 ] = UTCTimeString[ 2 ];
   temp_number[ 1 ] = UTCTimeString[ 3 ];

   int minutes = ::atoi( temp_number );

   temp_number[ 0 ] = UTCTimeString[ 4 ];
   temp_number[ 1 ] = UTCTimeString[ 5 ];

   int seconds = ::atoi( temp_number );

   UTCTime = CTime( year, month, day, hours, minutes, seconds );

   ArrivalTimeString = sentence.Field( 2 );

   temp_number[ 0 ] = ArrivalTimeString[ 0 ];
   temp_number[ 1 ] = ArrivalTimeString[ 1 ];

   hours = ::atoi( temp_number );

   temp_number[ 0 ] = ArrivalTimeString[ 2 ];
   temp_number[ 1 ] = ArrivalTimeString[ 3 ];

   minutes = ::atoi( temp_number );

   temp_number[ 0 ] = ArrivalTimeString[ 4 ];
   temp_number[ 1 ] = ArrivalTimeString[ 5 ];

   seconds = ::atoi( temp_number );

   ArrivalTime = CTime( year, month, day, hours, minutes, seconds );

   To = sentence.Field( 3 );

   return( TRUE );
}

BOOL ZWP::Write( SENTENCE& sentence )
{
   /*
   ** Let the parent do its thing
   */
   
   RESPONSE::Write( sentence );

   sentence += UTCTimeString;
   sentence += ArrivalTimeString;
   sentence += To;

   sentence.Finish();

   return( TRUE );
}

const ZWP& ZWP::operator = ( const ZWP& source )
{
   UTCTimeString     = source.UTCTimeString;
   UTCTime           = source.UTCTime;
   ArrivalTimeString = source.ArrivalTimeString;
   ArrivalTime       = source.ArrivalTime;
   To                = source.To;

   return( *this );
}
